@@ -1,7 +1,7 @@
 //
 // Job processing (printing) functions for the Printer Application Framework
 //
-// Copyright © 2019-2024 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -45,7 +45,7 @@ papplJobCreatePrintOptions(
   pappl_printer_t	*printer = job->printer;
 					// Printer
   const char		*raster_type;	// Raster type for output
-#if CUPS_VERSION_MAJOR < 3
+#if CUPS_VERSION_MAJOR < 3 && CUPS_VERSION_MINOR < 5
   static const char * const media_positions[] =
   {					// "media-source" to MediaPosition mapping
     "auto",
@@ -107,7 +107,7 @@ papplJobCreatePrintOptions(
     CUPS_ORIENT_180,
     CUPS_ORIENT_0
   };
-#endif // CUPS_VERSION_MAJOR < 3
+#endif // CUPS_VERSION_MAJOR < 3 && CUPS_VERSION_MINOR < 5
   static const char * const sheet_back[] =
   {					// "pwg-raster-document-sheet-back values
     "normal",
@@ -538,7 +538,7 @@ _papplJobProcess(pappl_job_t *job)	// I - Job
       if (!(filter->cb)(job, job->printer->device, filter->cbdata))
 	job->state = IPP_JSTATE_ABORTED;
     }
-    else if (!strcmp(job->format, job->printer->driver_data.format))
+    else if (job->printer->driver_data.format && job->printer->driver_data.printfile_cb && !strcmp(job->format, job->printer->driver_data.format))
     {
       if (!filter_raw(job, job->printer->device))
 	job->state = IPP_JSTATE_ABORTED;
@@ -641,10 +641,9 @@ _papplJobProcessRaster(
     if (job->is_canceled)
       break;
 
-    page ++;
     papplJobSetImpressionsCompleted(job, 1);
 
-    papplLogJob(job, PAPPL_LOGLEVEL_INFO, "Page %u raster data is %ux%ux%u (%s)", page, header.cupsWidth, header.cupsHeight, header.cupsBitsPerPixel, cups_cspace_string(header.cupsColorSpace));
+    papplLogJob(job, PAPPL_LOGLEVEL_INFO, "Page %u raster data is %ux%ux%u (%s)", page + 1, header.cupsWidth, header.cupsHeight, header.cupsBitsPerPixel, cups_cspace_string(header.cupsColorSpace));
 
     papplSystemAddEvent(printer->system, printer, job, PAPPL_EVENT_JOB_PROGRESS, NULL);
 
@@ -820,6 +819,8 @@ _papplJobProcessRaster(
       break;
     }
 
+    page ++;
+
     if (page == next_copy)
     {
       // Report a completed copy...
@@ -1114,7 +1115,7 @@ finish_job(pappl_job_t  *job)		// I - Job
   if (papplPrinterIsDeleted(printer))
   {
     papplPrinterDelete(printer);
-    printer = NULL;
+    return;
   }
   else if (!strncmp(printer->device_uri, "file:", 5) || cupsArrayGetCount(printer->active_jobs) == 0)
   {
