@@ -1,7 +1,7 @@
 //
 // Job object for the Printer Application Framework
 //
-// Copyright © 2019-2024 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
@@ -511,7 +511,7 @@ papplJobOpenFile(
   if (!strcmp(mode, "r"))
     return (open(fname, O_RDONLY | O_NOFOLLOW | O_CLOEXEC | O_BINARY));
   else if (!strcmp(mode, "w"))
-    return (open(fname, O_WRONLY | O_CREAT | O_TRUNC | O_NOFOLLOW | O_CLOEXEC | O_BINARY, 0600));
+    return (open(fname, O_WRONLY | O_CREAT | O_TRUNC | O_NOFOLLOW | O_CLOEXEC | O_BINARY, 0400));
   else if (!strcmp(mode, "x"))
     return (unlink(fname));
   else
@@ -793,6 +793,8 @@ _papplJobSubmitFile(
     pappl_job_t *job,			// I - Job
     const char  *filename)		// I - Filename
 {
+  _papplRWLockWrite(job);
+
   if (!job->format)
   {
     // Open the file
@@ -862,10 +864,18 @@ _papplJobSubmitFile(
       // Process the job...
       job->state = IPP_JSTATE_PENDING;
 
+      _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_STATE_CHANGED, NULL);
+
+      _papplRWUnlock(job);
       _papplRWLockWrite(job->printer);
       _papplPrinterCheckJobsNoLock(job->printer);
       _papplRWUnlock(job->printer);
     }
+    else
+    {
+      // Queue the job...
+      _papplRWUnlock(job);
+    }
   }
   else
   {
@@ -876,6 +886,10 @@ _papplJobSubmitFile(
     job->state     = IPP_JSTATE_ABORTED;
     job->completed = time(NULL);
 
+    _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_COMPLETED, "Job aborted.");
+
+    _papplRWUnlock(job);
+
     papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to allocate filename.");
 
     if (!strncmp(filename, job->system->directory, dirlen) && filename[dirlen] == '/')
