@@ -1,7 +1,7 @@
 //
 // Standard papplMainloop sub-commands for the Printer Application Framework
 //
-// Copyright © 2020-2024 by Michael R Sweet.
+// Copyright © 2020-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -1135,6 +1135,7 @@ _papplMainloopShowOptions(
   print_option(response, "media-tracking");
   print_option(response, "media-type");
   print_option(response, "orientation-requested");
+  print_option(response, "output-bin");
   print_option(response, "print-color-mode");
   print_option(response, "print-content-optimize");
   if (ippFindAttribute(response, "print-darkness-supported", IPP_TAG_ZERO))
@@ -1621,7 +1622,7 @@ default_system_cb(
     void          *data)		// I - Data (unused)
 {
   pappl_system_t *system;		// System object
-  pappl_soptions_t soptions = PAPPL_SOPTIONS_MULTI_QUEUE | PAPPL_SOPTIONS_WEB_INTERFACE;
+  pappl_soptions_t soptions = PAPPL_SOPTIONS_MULTI_QUEUE | PAPPL_SOPTIONS_WEB_INTERFACE | PAPPL_SOPTIONS_WEB_TLS;
 					// Server options
   char		spoolname[1024];	// Default spool directory
   const char	*directory = cupsGetOption("spool-directory", (cups_len_t)num_options, options),
@@ -1689,7 +1690,7 @@ default_system_cb(
       else if (!strcmp(valptr, "web-security") || !strncmp(valptr, "web-security,", 13))
         soptions |= PAPPL_SOPTIONS_WEB_SECURITY;
       else if (!strcmp(valptr, "no-tls") || !strncmp(valptr, "no-tls,", 7))
-        soptions |= PAPPL_SOPTIONS_NO_TLS;
+        soptions = (pappl_soptions_t)((soptions | PAPPL_SOPTIONS_NO_TLS) & (pappl_soptions_t)~PAPPL_SOPTIONS_WEB_TLS);
 
       if ((valptr = strchr(valptr, ',')) != NULL)
         valptr ++;
@@ -1945,7 +1946,7 @@ load_options(const char    *filename,	// I  - Filename
       continue;
 
     // Parse any options on this line...
-    num_loptions = cupsParseOptions(line, 0, &loptions);
+    num_loptions = cupsParseOptions(line, /*end*/NULL, 0, &loptions);
 
     // Copy any unset line options to the options array...
     for (i = num_loptions, loption = loptions; i > 0; i --, loption ++)
