@@ -1,7 +1,7 @@
 //
 // System accessor functions for the Printer Application Framework
 //
-// Copyright © 2020-2024 by Michael R Sweet.
+// Copyright © 2020-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -132,10 +132,22 @@ papplSystemAddListeners(
   else
   {
     // Add named listeners on both IPv4 and IPv6...
+    if (name && strcasecmp(name, "*"))
+    {
+      // Listening on a specific hostname...
+      _papplRWLockWrite(system);
+
+      free(system->hostname);
+      system->hostname      = strdup(name);
+      system->is_listenhost = true;
+
+      _papplRWUnlock(system);
+    }
+
     if (system->port)
     {
-      ret = add_listeners(system, name, system->port, AF_INET) ||
-            add_listeners(system, name, system->port, AF_INET6);
+      ret =  add_listeners(system, name, system->port, AF_INET);
+      ret |= add_listeners(system, name, system->port, AF_INET6);
     }
     else
     {
@@ -157,7 +169,7 @@ papplSystemAddListeners(
       if (ret)
       {
         system->port = port;
-        add_listeners(system, name, port, AF_INET6);
+        ret |= add_listeners(system, name, port, AF_INET6);
       }
     }
   }
@@ -1277,7 +1289,7 @@ papplSystemHashPassword(
     {
       // Copy existing nonce from the salt string...
       papplCopyString(nonce, salt, sizeof(nonce));
-      if ((ptr = strchr(nonce, ':')) != NULL)
+      if ((ptr = strchr(nonce, '~')) != NULL)
         *ptr = '\0';
     }
     else
@@ -1323,6 +1335,21 @@ papplSystemIsRunning(
 }
 
 
+//
+// '_papplSystemIsShutdownNoLock()' - Return whether the system has been shutdown.
+//
+// This function returns whether the system is shutdown or scheduled to
+// shutdown.
+//
+
+bool					// O - `true` if the system is shutdown, `false` otherwise
+_papplSystemIsShutdownNoLock(
+    pappl_system_t *system)		// I - System
+{
+  return (!system->is_running || system->shutdown_time != 0);
+}
+
+
 //
 // 'papplSystemIsShutdown()' - Return whether the system has been shutdown.
 //
@@ -1340,7 +1367,7 @@ papplSystemIsShutdown(
   if (system)
   {
     _papplRWLockRead(system);
-    ret = !system->is_running || system->shutdown_time != 0;
+    ret = _papplSystemIsShutdownNoLock(system);
     _papplRWUnlock(system);
   }
 
@@ -1373,10 +1400,10 @@ papplSystemIteratePrinters(
   // Note: Cannot use cupsArrayGetFirst/Last since other threads might be
   // enumerating the printers array.
 
-  _papplRWLockRead(system);
+  pthread_rwlock_rdlock(&system->printers_rwlock);
   for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
     (cb)((pappl_printer_t *)cupsArrayGetElement(system->printers, i), data);
-  _papplRWUnlock(system);
+  pthread_rwlock_unlock(&system->printers_rwlock);
 }
 
 
@@ -1570,6 +1597,9 @@ papplSystemSetAdminGroup(
 // - `HTTP_STATUS_FORBIDDEN` if the authentication succeeded but the user is
 //   not part of the specified group.
 //
+// > Note: The authentication callback can only be set prior to calling
+// > @link papplSystemRun@.
+//
 
 void
 papplSystemSetAuthCallback(
@@ -1578,16 +1608,12 @@ papplSystemSetAuthCallback(
     pappl_auth_cb_t auth_cb,		// I - Callback function
     void            *auth_cbdata)	// I - Callback data
 {
-  if (system)
+  if (system && !system->is_running)
   {
-    _papplRWLockWrite(system);
-
     free(system->auth_scheme);
     system->auth_scheme = auth_scheme ? strdup(auth_scheme) : NULL;
     system->auth_cb     = auth_cb;
     system->auth_cbdata = auth_cbdata;
-
-    _papplRWUnlock(system);
   }
 }
 
@@ -1742,12 +1768,8 @@ papplSystemSetFooterHTML(
 {
   if (system && html && !system->is_running)
   {
-    _papplRWLockWrite(system);
-
     free(system->footer_html);
     system->footer_html = strdup(html);
-
-    _papplRWUnlock(system);
   }
 }
 
@@ -1821,6 +1843,9 @@ _papplSystemSetHostNameNoLock(
 	*ptr;				// Pointer in temporary hostname
 
 
+  if (system->is_listenhost)
+    return;
+
   if (value)
   {
 #if !defined(__APPLE__) && !_WIN32
