@@ -1,17 +1,13 @@
 //
 // IPP processing for the Printer Application Framework
 //
-// Copyright © 2019-2023 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -456,6 +452,8 @@ ipp_disable_all_printers(
 {
   pappl_system_t	*system = client->system;
 					// System
+  cups_len_t		i,		// Looping var
+			count;		// Number of printers
   pappl_printer_t	*printer;	// Current printer
   http_status_t		auth_status;	// Authorization status
 
@@ -468,12 +466,14 @@ ipp_disable_all_printers(
   }
 
   // Loop through the printers...
-  _papplRWLockRead(system);
-  for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
+  pthread_rwlock_rdlock(&system->printers_rwlock);
+  for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
   {
+    printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
+
     papplPrinterDisable(printer);
   }
-  _papplRWUnlock(system);
+  pthread_rwlock_unlock(&system->printers_rwlock);
 }
 
 
@@ -487,6 +487,8 @@ ipp_enable_all_printers(
 {
   pappl_system_t	*system = client->system;
 					// System
+  cups_len_t		i,		// Looping var
+			count;		// Number of printers
   pappl_printer_t	*printer;	// Current printer
   http_status_t		auth_status;	// Authorization status
 
@@ -499,12 +501,14 @@ ipp_enable_all_printers(
   }
 
   // Loop through the printers...
-  _papplRWLockRead(system);
-  for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
+  pthread_rwlock_rdlock(&system->printers_rwlock);
+  for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
   {
+    printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
+
     papplPrinterEnable(printer);
   }
-  _papplRWUnlock(system);
+  pthread_rwlock_unlock(&system->printers_rwlock);
 }
 
 
@@ -783,7 +787,7 @@ ipp_get_printers(
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
-  _papplRWLockRead(system);
+  pthread_rwlock_rdlock(&system->printers_rwlock);
 
   // Enumerate the printers for the client...
   count = cupsArrayGetCount(system->printers);
@@ -806,7 +810,7 @@ ipp_get_printers(
     _papplRWUnlock(printer);
   }
 
-  _papplRWUnlock(system);
+  pthread_rwlock_unlock(&system->printers_rwlock);
 
   cupsArrayDelete(ra);
 }
@@ -838,8 +842,9 @@ ipp_get_system_attributes(
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
   _papplRWLockRead(system);
+  pthread_rwlock_rdlock(&system->printers_rwlock);
 
-  _papplCopyAttributes(client->response, system->attrs, ra, IPP_TAG_ZERO, IPP_TAG_CUPS_CONST);
+  _papplCopyAttributes(client->response, system->attrs, ra, IPP_TAG_ZERO, true);
 
   if (!ra || cupsArrayFind(ra, "system-config-change-date-time") || cupsArrayFind(ra, "system-config-change-time"))
   {
@@ -1006,6 +1011,7 @@ ipp_get_system_attributes(
     ippDelete(col);
   }
 
+  pthread_rwlock_unlock(&system->printers_rwlock);
   _papplRWUnlock(system);
 
   cupsArrayDelete(ra);
@@ -1022,6 +1028,8 @@ ipp_pause_all_printers(
 {
   pappl_system_t	*system = client->system;
 					// System
+  cups_len_t		i,		// Looping var
+			count;		// Number of printers
   pappl_printer_t	*printer;	// Current printer
   http_status_t		auth_status;	// Authorization status
 
@@ -1034,12 +1042,14 @@ ipp_pause_all_printers(
   }
 
   // Loop through the printers...
-  _papplRWLockRead(system);
-  for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
+  pthread_rwlock_rdlock(&system->printers_rwlock);
+  for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
   {
-    papplPrinterPause(client->printer);
+    printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
+
+    papplPrinterPause(printer);
   }
-  _papplRWUnlock(system);
+  pthread_rwlock_unlock(&system->printers_rwlock);
 }
 
 
@@ -1053,6 +1063,8 @@ ipp_resume_all_printers(
 {
   pappl_system_t	*system = client->system;
 					// System
+  cups_len_t		i,		// Looping var
+			count;		// Number of printers
   pappl_printer_t	*printer;	// Current printer
   http_status_t		auth_status;	// Authorization status
 
@@ -1065,12 +1077,14 @@ ipp_resume_all_printers(
   }
 
   // Loop through the printers...
-  _papplRWLockRead(system);
-  for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
+  pthread_rwlock_rdlock(&system->printers_rwlock);
+  for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
   {
-    papplPrinterResume(client->printer);
+    printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
+
+    papplPrinterResume(printer);
   }
-  _papplRWUnlock(system);
+  pthread_rwlock_unlock(&system->printers_rwlock);
 }
 
 
