@@ -1,7 +1,7 @@
 //
 // System load/save functions for the Printer Application Framework
 //
-// Copyright © 2020-2024 by Michael R Sweet.
+// Copyright © 2020-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -137,7 +137,7 @@ papplSystemLoadState(
       pappl_printer_t	*printer;	// Current printer
 
 
-      if ((num_options = cupsParseOptions(value, 0, &options)) == 0 || (printer_id = cupsGetOption("id", num_options, options)) == NULL || strtol(printer_id, NULL, 10) <= 0 || (printer_name = cupsGetOption("name", num_options, options)) == NULL || (device_id = cupsGetOption("did", num_options, options)) == NULL || (device_uri = cupsGetOption("uri", num_options, options)) == NULL || (driver_name = cupsGetOption("driver", num_options, options)) == NULL)
+      if ((num_options = cupsParseOptions(value, /*end*/NULL, 0, &options)) == 0 || (printer_id = cupsGetOption("id", num_options, options)) == NULL || strtol(printer_id, NULL, 10) <= 0 || (printer_name = cupsGetOption("name", num_options, options)) == NULL || (device_id = cupsGetOption("did", num_options, options)) == NULL || (device_uri = cupsGetOption("uri", num_options, options)) == NULL || (driver_name = cupsGetOption("driver", num_options, options)) == NULL)
       {
         papplLog(system, PAPPL_LOGLEVEL_ERROR, "Bad printer definition on line %d of '%s'.", linenum, filename);
         break;
@@ -288,7 +288,7 @@ papplSystemLoadState(
 			*job_format,	// Job format
 			*job_value;	// Job option value
 
-	  num_options = cupsParseOptions(value, 0, &options);
+	  num_options = cupsParseOptions(value, /*end*/NULL, 0, &options);
 
 	  if ((job_id = cupsGetOption("id", num_options, options)) == NULL || strtol(job_id, NULL, 10) <= 0 || (job_name = cupsGetOption("name", num_options, options)) == NULL || (job_username = cupsGetOption("username", num_options, options)) == NULL || (job_format = cupsGetOption("format", num_options, options)) == NULL)
 	  {
@@ -443,6 +443,8 @@ papplSystemSaveState(
   //
   // Note: Cannot use cupsArrayGetFirst/Last since other threads might be
   // enumerating the printers array.
+  pthread_rwlock_rdlock(&system->printers_rwlock);
+
   for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
   {
     cups_len_t		jcount;		// Number of jobs
@@ -620,6 +622,7 @@ papplSystemSaveState(
     _papplRWUnlock(printer);
   }
 
+  pthread_rwlock_unlock(&system->printers_rwlock);
   _papplRWUnlock(system);
 
   cupsFileClose(fp);
@@ -664,7 +667,7 @@ parse_contact(char            *value,	// I - Value
 
 
   memset(contact, 0, sizeof(pappl_contact_t));
-  num_options = cupsParseOptions(value, 0, &options);
+  num_options = cupsParseOptions(value, /*end*/NULL, 0, &options);
 
   for (i = num_options, option = options; i > 0; i --, option ++)
   {
@@ -696,7 +699,7 @@ parse_media_col(
 
 
   memset(media, 0, sizeof(pappl_media_col_t));
-  num_options = cupsParseOptions(value, 0, &options);
+  num_options = cupsParseOptions(value, /*end*/NULL, 0, &options);
 
   for (i = num_options, option = options; i > 0; i --, option ++)
   {
