@@ -336,6 +336,7 @@ papplSystemDelete(
   cupsArrayDelete(system->timers);
 
   pthread_rwlock_destroy(&system->rwlock);
+  pthread_rwlock_destroy(&system->printers_rwlock);
   pthread_mutex_destroy(&system->session_mutex);
   pthread_mutex_destroy(&system->config_mutex);
   pthread_mutex_destroy(&system->log_mutex);
@@ -401,8 +402,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
   int			pcount,		// Poll count
 			ptimeout;	// Poll timeout
   pappl_client_t	*client;	// New client
-  char			header[HTTP_MAX_VALUE];
-					// Server: header value
+  char			header[256];	// Server: header value
   int			dns_sd_host_changes;
 					// Current number of host name changes
   pappl_printer_t	*printer;	// Current printer
@@ -537,6 +537,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
     _papplSystemRegisterDNSSDNoLock(system);
 
   // Start up printers...
+  pthread_rwlock_rdlock(&system->printers_rwlock);
   for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
   {
     printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
@@ -566,13 +567,18 @@ papplSystemRun(pappl_system_t *system)	// I - System
 	while (!printer->raw_active)
 	{
 	  _papplRWUnlock(printer);
+          pthread_rwlock_unlock(&system->printers_rwlock);
+
 	  usleep(1000);			// Wait for raw thread to start
+
+          pthread_rwlock_rdlock(&system->printers_rwlock);
 	  _papplRWLockRead(printer);
 	}
 	_papplRWUnlock(printer);
       }
     }
   }
+  pthread_rwlock_unlock(&system->printers_rwlock);
 
   // Start the USB gadget as needed...
   if ((system->options & PAPPL_SOPTIONS_USB_PRINTER) && (printer = papplSystemFindPrinter(system, NULL, system->default_printer_id, NULL)) != NULL)
@@ -702,6 +708,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
       if (system->dns_sd_collision || force_dns_sd)
         _papplSystemRegisterDNSSDNoLock(system);
 
+      pthread_rwlock_rdlock(&system->printers_rwlock);
       for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
       {
 	printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
@@ -709,6 +716,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
         if (printer->dns_sd_collision || force_dns_sd)
           _papplPrinterRegisterDNSSDNoLock(printer);
       }
+      pthread_rwlock_unlock(&system->printers_rwlock);
 
       system->dns_sd_any_collision = false;
       system->dns_sd_host_changes  = dns_sd_host_changes;
@@ -750,6 +758,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
       }
 
       // Otherwise shutdown immediately if there are no more active jobs...
+      pthread_rwlock_rdlock(&system->printers_rwlock);
       for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
       {
 	printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
@@ -809,6 +818,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
   if (system->dns_sd_name)
     _papplSystemUnregisterDNSSDNoLock(system);
 
+  pthread_rwlock_rdlock(&system->printers_rwlock);
   for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
   {
     printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
@@ -817,6 +827,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
     if (printer->dns_sd_name)
       _papplPrinterUnregisterDNSSDNoLock(printer);
   }
+  pthread_rwlock_unlock(&system->printers_rwlock);
 
   system->is_running = false;
 
